// gameobject.cpp
#include "gameobject.h"
#include "manager.h"
#include "constants.h"
#include "graphics_offsets.h"
#include "playercharacter.h"

#include <alleggl.h>

StaticObject::StaticObject(Manager *m, StaticObjectType t) : SpriteObject(m), type(t)
{
	spriteSet = ResourceManager::SpriteSetMisc;
	spriteScale = MiscSpriteScale;
	spriteOffset = - MiscSpriteFloorLevel;
	switch (type) {
		case XChest:
			spriteNum = ObjectChestX;
			break;
		case YChest:
			spriteNum = ObjectChestY;
			break;
		case Vase:
			spriteNum = ObjectVase;
			spriteOffset = spriteOffset + 0.5;
			break;
		case Skull:
			spriteNum = ObjectSkull;
			gettable = true;
			break;
		case SkullPile:
			spriteNum = ObjectSkulls;
			gettable = true;
			break;
		case Goat:
			spriteNum = ObjectGoat;
			gettable = true;
			// The goat is far too big.
			spriteOffset = spriteOffset + 0.5;
			break;
		default: 
			spriteNum = 0;
			break;
	}
}

void StaticObject::gotBy(PlayerCharacter *p)
{
	// The player got me.
	dead = true;
	switch (type) {
		case Skull:
			p->addAmmo(1);
			break;
		case SkullPile:
			p->addAmmo(6);
			break;
		case Goat:
			p->addHealth(50);
			break;
		default: 
			break;
	}
}

void SpriteObject::locationChanged()
{
	GameObject::locationChanged();
	// Make sure we're on the ground.
	location.z = 0;
}

void SpriteObject::render()
{
	// Render a sprite
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND); // Enable Blending
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	glColor3f(1,1,1);
	manager->rm.selectSprite(spriteSet, spriteNum);
	glBegin(GL_TRIANGLE_STRIP);
	float leftx = (location.x - spriteScale);
	float lefty = (location.y + spriteScale);
	float rightx = (location.x + spriteScale);
	float righty = (location.y - spriteScale);
	float bottomz = location.z + spriteOffset;
	float topz = bottomz + (spriteScale * 2 * RootTwo);
	glTexCoord2f(0,0);
	glVertex3f(leftx, lefty, topz);
	glTexCoord2f(0,1);
	glVertex3f(leftx, lefty, bottomz);
	glTexCoord2f(1,0);
	glVertex3f(rightx, righty, topz);
	glTexCoord2f(1,1);
	glVertex3f(rightx, righty, bottomz);
	glEnd();
	
	if (ShowDebugGfx) {
		// A blob for debugging purposes.
		/* Render blob */
		glDisable(GL_TEXTURE_2D);
		glColor3f(1,1,1);
		glPointSize(5.0);
		// Render it at the actual position.
		glBegin(GL_POINTS);
		glVertex3f(location.x, location.y, location.z);
		glEnd();
	}
}

void SpriteObject::runLogic()
{
	/* nothing. */
}

void GameObject::handleCollision(const Vect3d & overlap, GameObject *what)
{
	if (what->solid) {
		// Move the object away by half the overlap.
		Vect3d halfOverlap(overlap * 0.5);
		location = location + halfOverlap;
		locationChanged();
	}
}
