#include "manager.h"
#include "playercharacter.h"

#include <cstring>
#include <algorithm>
#include <cstdio>
 
Manager::Manager()
{
	// Load the resources.
	rm.load();
	// Init the map up (ultimately broken out into levels)
	mapCentre = Vect3d(5,5,0);
	map.manager = this;
	map.loadEmptyMap();
	currentLayer = 0;
	trackObject = 0;
	player = 0;
	levelNum = 1;
}

void Manager::drawTextAt(const char * msg, 
	float x, float y, float z)
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(x,y,z);
	/* main.cpp does this :
		glRotatef(-60, 1, 0, 0); // rotate -60 deg about x axis
		glRotatef(45.0, 0, 0, 1); // rotate 45 deg about z.
	
	We essentially undo it:*/
	
	glRotatef(-45.0,0,0,1);
	glRotatef(60.0,1,0,0);
	int colour = makecol(255,255,255);
	int len = std::strlen(msg);
	float xoffset = - ((float) len / 4.0);
	allegro_gl_printf(rm.getMainFont(),  // font
		xoffset, 0, 0, // x,y,z
		colour, // colour
		"%s", msg);
	glPopMatrix();

}

void Manager::render()
{
	// Sort out the transformation matrix
	// Based on the position of our heroine..
	glTranslatef(- mapCentre.x, - mapCentre.y, - mapCentre.z);
	// Render the map...
	renderMap();
	// Render some text.
	// drawTextAt("Hello", 0,0,3);
}

/*
 * The manager needs to take care of rendering everything in the right order.
 *
 * This is because we're not using z-buffer, we have to render from back to front.
 */

void Manager::renderMap()
{
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	
	Component centrexf = mapCentre.x;
	Component centreyf = mapCentre.y;
	centrexf -= 0.5;
	// centreyf -= 0.25;
	int centrex = (int) centrexf;
	int centrey = (int) centreyf;
	// Determine which tiles to render
	// Start at the back (= + x, + y)
	// Move to the front (-x, -y)
	// Diagonally
	// Rendering a diamond shape on the map.
	const int DiagonalH = 10;
	const int DiagonalV = 16;
	const int HalfH = DiagonalH / 2;
	const int QuarterH = HalfH / 2;
	const int HalfV = DiagonalV / 2;
	// r = row; c = column. (as opposed to x and y which go iso)
	for (int r = 0; r < DiagonalV; r++)
	{
		int startx = centrex + HalfV;
		int starty = centrey + HalfV;
		// Offset a 1/2 to the left (from the VP)
		startx -= QuarterH + 2;
		starty += QuarterH + 2;
		// Slight extra adjustment. 
		starty = starty - 1;
		startx = startx - 1;
		// Subtract the line number.
		startx -= r;
		starty -= r;
		// MOve left to right.
		for (int c = 0; c < DiagonalH; c++) {
			// Moving to the +X and -Y dirs, diagonally.
			int x = startx + c;
			int y = starty - c;
			renderBlock(x,y);
		}
		// Same, but one place in the -x direction.
		startx --;
		for (int c = 0; c < DiagonalH; c++) {
			// Moving to the +X and -Y dirs, diagonally.
			int x = startx + c;
			int y = starty - c;
			renderBlock(x,y);
		}
	}
	// Render them.
}

void Manager::renderBlock(int x, int y)
{
	// This function will ultimately render the contents of the cell, including
	// any objects (even the player).
	// this will respect which layer we're in.
	bool visible = map.renderBlock(x,y,currentLayer);
	
	if (visible) {
		for(GameObjectIterator i = liveObjects.begin(); i != liveObjects.end(); i++) {
			GameObject *obj = *i;
			if (obj->getBlockX() == x) {
				if (obj->getBlockY() == y) {
					if (obj->getLayer() == currentLayer) {
						obj->render();
					}
				}
			}
		}
	}
}

void Manager::runLogic()
{
	/*
	if (key[KEY_RIGHT]) {
		mapCentre.x += 0.02;
	}
	if (key[KEY_LEFT]) {
		mapCentre.x -= 0.02;
	}
	if (key[KEY_UP]) {
		mapCentre.y += 0.02;
	}
	if (key[KEY_DOWN]) {
		mapCentre.y -= 0.02;
	}
	*/
	map.runLogic();
	// Add new ones...
	for(GameObjectIterator i = newObjects.begin(); i != newObjects.end(); i++) {
                GameObject *obj = *i;
		liveObjects.push_back(obj);
        }
	newObjects.clear();
	// Tick teh objcets.
	
	for(GameObjectIterator i = liveObjects.begin(); i != liveObjects.end(); i++) {
		GameObject * obj = *i;
		obj->runLogic();
	}
	checkCollisions();
	clearDeads();
	if (trackObject) {
		mapCentre = trackObject->getLocation();
	}
	
}
void Manager::initObjects()
{
	clearObjects(liveObjects);
	clearObjects(deadObjects);
	
	// Create the player object...
	player = new PlayerCharacter(this);
	player->setLocation(Vect3d(4.5,8.5,0));
	// player->getBlockX();
	addObject(player);
	trackObject = player;
	
	// Create some test objects
	/*
	GameObject *testobj = new StaticObject(this, StaticObject::XChest);
	testobj->setLocation(Vect3d(4.5,11.5,0));
	addObject(testobj);
	testobj = new StaticObject(this, StaticObject::Vase);
	testobj->setLocation(Vect3d(4.5,13.5,0));
	addObject(testobj);
	testobj = new StaticObject(this, StaticObject::YChest);
	testobj->setLocation(Vect3d(4.5,12.5,0));
	addObject(testobj);
	*/
	levelNum = 1;
	initLevel();
}

void Manager::initLevel()
{
	// Delete anything except the player from the map.
	for(GameObjectIterator i = liveObjects.begin(); i != liveObjects.end(); i++) {
		GameObject * obj = *i;
		if (obj != player) {
			obj->setDead(true);
		}
	}
	clearDeads();
	char filename[50];
	std::sprintf(filename, "levels/level%d.txt", levelNum);
	// Load the map (will relocate the player)
	map.load(this, filename);
	
	// Make sure the area around the player is visible
	Vect3d playerpos = player->getLocation();
	int z = player->getLayer();
	map.makeTileVisible((int) playerpos.x, (int) playerpos.y, z);
	levelComplete = false;
}

static bool isDead(GameObject *obj) {
	return obj->isDead();
}

void Manager::clearDeads()
{
	clearObjects(deadObjects);
	// Now trash any dead ones...
	for(GameObjectIterator i = liveObjects.begin(); i != liveObjects.end(); i++) {
		GameObject * obj = *i;
		if (obj->isDead()) deadObjects.push_back(obj);
	}
	// Wipe them out.
	GameObjectIterator new_end =
		std::remove_if(liveObjects.begin(), liveObjects.end(),isDead);
	liveObjects.erase(
		new_end,
		liveObjects.end());
}

void Manager::addObject(GameObject *obj)
{
	// Add it to the new list.
	newObjects.push_back(obj);
}

void Manager::clearObjects(GameObjectList &list)
{
	for (GameObjectIterator i = list.begin(); i != list.end(); i++) {
		GameObject * obj = *i;
		delete obj;
	}
	list.clear();
}

Manager::~Manager()
{
	clearObjects(liveObjects);
	clearObjects(deadObjects);
}

void Manager::handleClick(Component x, Component y)
{
	if (player) {
		player->setTarget(Vect3d(x,y));
	}
}
void Manager::handleAttack(Component x, Component y)
{
	if (player) {
		player->beginAttack(Vect3d(x,y));
	}
}

void Manager::selectWeapon(int w)
{
	if (player) {
		player->selectWeapon((PlayerCharacter::Weapon) (w - 1));
	}
}

		
void Manager::relocatePlayer(Vect3d where, int layer)
{
	if (player) {
		player->setLocation(where);
		player->setLayer(layer);
	}
}

void Manager::collideTest(GameObject *first, GameObject *second)
{
	// They can only collide if they're in the same layer
	if (first->getLayer() != second->getLayer()) {
		return;
	}
	Vect3d firstToSecond = second->getLocation() - first->getLocation();
	Component radiusSum = first->getRadius() + second->getRadius();
	if (firstToSecond.magnitudeSquared() < (radiusSum * radiusSum)) {
		// Handle a collision.
		// Find a vector of length radiusSum, in the direction of firstToSecond
		Vect3d desiredDisplacement = (firstToSecond * (1 / firstToSecond.magnitude())) * radiusSum;
		// The overlap, is the difference between the 
		// desired Displacement, and the actual displacement.
		Vect3d overlap = desiredDisplacement - firstToSecond;
		// This will be in the direction of first -> second 
		first->handleCollision(- overlap, second);
		second->handleCollision(overlap, first);
	}
}

void Manager::checkCollisions()
{
	// Check every pair of collidable objects
	// TODO: use some optimisation for this in the future.
	for (GameObjectIterator i = liveObjects.begin(); i != liveObjects.end(); i++) {
		GameObjectIterator j = i;
		j++;
		GameObject *first = *i;
		if (first->isCollidable()) {
			for (; j != liveObjects.end(); j++) {
				GameObject *second = *j;
				if (second->isCollidable()) {
					// Determine whether they collided?
					collideTest(first, second);
				}
			}
		}
	}
}

// Causes amount of damage to anyone within the radius
// EXCEPT the object to avoid.
void Manager::areaDamage(const Vect3d &where, Component amount, Component radius, GameObject *avoid)
{
	for (GameObjectIterator i = liveObjects.begin(); i != liveObjects.end(); i++) {
		GameObject *obj = *i;
		if (obj->isCollidable() && (obj != avoid)) {
			Component radiusSum = obj->getRadius() + radius;
			Vect3d rel = obj->getLocation() - where;
			if (rel.magnitudeSquared() < (radiusSum * radiusSum)) {
				obj->damage(amount);
			}
		}
	}
}

